//
//	test the EMOS redistribution retrieval
//
#include <iostream>
#include <cassert>
#include "CalServer.h"
#include "CalCommon.h"
#include "Redistribution.h"
#include "Xmm.h"

using namespace std;

int main(int argc, char **argv)
{
	//	setup the the Cal server
	const Xmm::Instrument instrument = Xmm::EMOS1;
	calServer.setInstrument(instrument);
	calServer.state()->ccd()->set(1);
	calServer.state()->mode()->set(Xmm::FAST_UNCOMPRESSED);
	calServer.state()->node()->set(EMOScam::PRIMARY_NODE);

	if (argc == 2)
		//	assume the argument is a REDIST replacement
		calServer.ccf()->registerNewCifEntry(argv[1]);

	//	get the rmf
	CanonicalRedist *rmf = 0; rmf = calServer.getAtom(rmf);
	rmf->at(300, 300);
	const BinnedAxis &ebins = rmf->eBins();

	unsigned nChans = ebins.bins();
	cout<<"RMF for intrument: "<<Xmm::toString(instrument)<<endl;
	cout<<"\nbinNr     binLo   binHi    peakChannel\n";
	for (unsigned n=0; n<nChans; n+=100) {
		Spectrum spec(rmf->spectrum(n));

		//	locate peak channel
		unsigned peakChannel = 0;
		double maxVal = -1.;
		unsigned len = spec.size();
		for (unsigned m=0; m<len; ++m)
			if (spec[m] > maxVal) {
				maxVal = spec[m];
				peakChannel = m;
			}

		unsigned rmfRowPeak = spec.peakChannel();
		cout<<n<<" "<<ebins.bin(n).lo()<<" "<<ebins.bin(n).hi()<<" "
			<<peakChannel<<" "<<rmfRowPeak<<endl;
		assert(peakChannel == rmfRowPeak);
	}

	errstr<<"CCF constituents accessed during CAL livetime:\n"
		  <<calServer.ccfAccessLog()<<endl
		  <<Sas::message(Msg::App, Msg::Always);

	return 0;
}







