!
!   psftest.f90  --- f90 program to test the PSF-related CAL routines
!
!   DESCRIPTION:
!       This tests a few aspects of the PSF-related CAL routines
!
program psftest

use cal             ! use the Calibration Access Layer
use dal
use errorhandling

    implicit none
    logical                        :: isClose
    integer, dimension(0:2)        :: insVec = (/EMOS1, EMOS2, EPN/)
    integer, dimension(0:3)        :: accVec = (/ACCURACY_LOW, &
                                                 ACCURACY_MEDIUM, &
                                                 ACCURACY_EXTENDED, &
                                                 ACCURACY_HIGH/)
    integer                        :: ins, acc, Ei, nx, ny, iter
    real(double), dimension(0:2)   :: Evec = (/1500, 4500, 8500/)
    real(double), dimension(0:26)  :: onval = (/ &
    0.4091759723441300, &
    0.3720914778616852, &
    0.3209737093831113, &
    0.5382853696743647, &
    0.4057884232865440, &
    0.2676861379985456, &
    0.3492290574527561, &
    0.3492290574527561, &
    0.3492290574527561, &
    0.4093759285804727, &
    0.3722808465156883, &
    0.3211405592937853, &
    0.5382853696743647, &
    0.4057884232865440, &
    0.2676861379985456, &
    0.2942574998318281, &
    0.2942574998318281, &
    0.2942574998318281, &
    0.4094092624793676, &
    0.3723124157836069, &
    0.3211683747211920, &
    0.5382853696743647, &
    0.4057884232865440, &
    0.2676861379985456, &
    0.2942628565316449, &
    0.2942628565316449, &
    0.2942628565316449/)
    real(double), dimension(0:26)  :: eeval = (/ &
    0.3610813334705640, &
    0.3610813334705640, &
    0.3610813334705640, &
    2.4824341676101280, &
    3.7462188347571024, &
    5.0100035019040767, &
    2.3470286675586665, &
    2.3470286675586665, &
    2.3470286675586665, &
    0.3610813334705640, &
    0.3610813334705640, &
    0.3610813334705640, &
    2.4824341676101280, &
    3.7462188347571024, &
    5.0100035019040767, &
    3.6559485013894615, &
    3.6559485013894615, &
    3.6559485013894615, &
    0.3610813334705640, &
    0.3610813334705640, &
    0.3610813334705640, &
    2.4824341676101280, &
    3.7462188347571024, &
    5.0100035019040767, &
    3.6559485013894615, &
    3.6559485013894615, &
    3.6559485013894615/)

    real(double)                   :: E, ee, onAxisVal
    type(PsfT)                     :: psf
    type(PsfValidityRangesT)       :: valRange
    type(PsfBinSizeT)              :: binSize
    type(PsfWindowT)               :: window
    logical                        :: binSizeChangeable
    real(single), dimension(:,:), pointer :: map

    iter = 0
    do ins=0, size(insVec)-1
        do acc=0, size(accVec)-1
            call CAL_setState(instrument=insVec(ins), &
                              accuracyLevel=accVec(acc))
            do Ei=0, size(Evec-1)-1
               E = Evec(Ei)
               write(*, *) 'Retrieving PSF for ', instId(insVec(ins)), &
                           ' acc=', acc, 'at E=', E, ' eV'
               psf = CAL_getPSF(E, 10._double, 0._double)
               valRange = CAL_psfValidityRanges()
               write(*, *) 'validity ranges: '
               write(*, *) 'eMin         : ', valRange%eMin
               write(*, *) 'eMax         : ', valRange%eMax
               write(*, *) 'thetaMin     : ', valRange%thetaMin
               write(*, *) 'thetaMax     : ', valRange%thetaMax
               write(*, *) 'phiMin       : ', valRange%phiMin
               write(*, *) 'phiMax       : ', valRange%phiMax
               onAxisVal = CAL_psfPointValue(psf, 0._double, 0._double)
               write(*, *) 'value on-axis: ', onAxisVal
               if (.not.isClose(onAxisVal, onval(iter))) &
                   ! call error('psftest', 'incorrect on-axis PSF value')
               binSize = CAL_psfBinSize(psf)
               binSizeChangeable = CAL_psfBinSizeChangeable(psf);
               write(*, *) 'bin sizes    : ', binSize%binX, binSize%binY, &
                                              binSizeChangeable
!               if (binSizeChangeable) then
!                  binSize%binX = .5*binSize%binX
!                  binSize%binY = .5*binSize%binY
!                  call CAL_psfSetBinSize(psf, binSize)
!                  binSize = CAL_psfBinSize(psf)
!               endif
               window%halfX = .1_double
               window%halfY = .1_double
               write(*, *) 'image centered on PSF barycenter - ', &
                            'extent: ', window%halfX, 'x', window%halfY, 'mm'
               call CAL_psfImage(psf, window, map)
               do nx=0, size(map, 1)-1
                   write(*, '(100F10.4, 1X)') &
                              (map(nx, ny), ny=0, size(map,2)-1)
               enddo
               call CAL_releaseMemory(map)

               ee = CAL_psfEncircledEnergy(psf, .95_double)
               write(*, *) '95% of the energy contained in circle with ', &
                           'radius', ee, ' mm'
               if (.not.isClose(ee, eeval(iter))) &
        !           call error('psftest', 'incorrect EE value')
               iter = iter + 1
            enddo
        enddo
    enddo      

    call message(AppMsg, Verbose, "all tests passed")
end program

 
logical function isClose(value, trueval)
use types

    real(double), intent(in)    :: value, trueval
    real(double)                :: absval, abstrueval

    isClose = .true.
    if (value * trueval .lt. 0.) isClose = .false.
    absval = abs(value)
     abstrueval = abs(trueval)
    if (absval .lt. .9999 * abstrueval .or. &
        absval .gt. 1.001 * abstrueval) isClose = .false.
end function isClose
