//-----------------------------------------------------------------------------
//
//						XMM Science Analysis System (SAS)
//					   (c) 1997-2010 European Space Agency
//
//-----------------------------------------------------------------------------
//
//	RgsBackgroundServer --- the RgsBackgroundServer class
//
//	DESCRIPTION:
//		A class 'RgsBackgroundServer' is defined which provides access to the
//		RGS template background data
//
//	CHANGE HISTORY:
//		RDS, 2004-07-02: 1.0.0: - first version
//
//	AUTHOR:
//     R.D. Saxton
//
//	REFERENCES:
//		[1]: Software Specification Document for the XMM Science Analysis
//			 Subsystem, M. Dahlem et. al.,
//		[2]: Calibration Data Access Handbook, C. Erd et al.
//
//	$Header: /sasbuild/repositories/sasdev/sas/packages/cal/src/RgsBackgroundServer.h,v 1.1 2004/07/22 15:08:20 rsaxton Exp $
//

#ifndef RGS_BACKGROUND_SERVER_H
#define RGS_BACKGROUND_SERVER_H

#include "Dal.h"
#include "Ccf.h"
#include "CalChild.h"
#include "CalAtomsBase.h"
#include "slatec++.h"
#include "Proxy.h"


//
//	---( RGS background class )---------------------------------------------------------------
//
class RgsBackgnd {
public:

    // Constructor
	RgsBackgnd(double exposure, unsigned order, double level,
             vector<int16> channel, vector<int> counts,
             vector<int16> quality, vector<float>areascal,
             vector<float>backscal);
		//
		//	PURPOSE:
    //    Methods to return the data for a background object
  const double exposure() const;
  const double level() const;
  const vector<int16> channel() const;
  const vector<int> counts() const;
  const vector<int16> quality() const;
  const vector<float> areascal() const;
  const vector<float> backscal() const;


 	private:
    double _exposure;
    unsigned _order;
    double _level;
    vector<int16> _channel;
    vector<int> _counts;
    vector<int16> _quality;
    vector<float> _areascal;
    vector<float> _backscal;

}; // RgsBackgnd


//
//	---( RGS background server class )---------------------------------------------------------------
//
class RgsBackgroundServer : public CalAtom,
     public virtual CalChild<CalAtomCollection<RgsBackgroundServer> > {
public:
      RgsBackgroundServer(DataSet *backset,
                          CalAtomCollection<RgsBackgroundServer> *parent = 0);
        //
        //  PURPOSE:
        //      Instantiate RgsBackgroundServer object
        //

          virtual ~RgsBackgroundServer();
        //
        //  PURPOSE:
        //      Destruct RgsBackgroundServer object
        //

          virtual void dump(std::ostream &os);
        //
        //  PURPOSE:
        //      Dump data members to an ostream
        //
        //  ARGUMENTS:
        //      In:
        //          os: stream to print on
        //

          static unsigned id() { return Ccf::TemplateBckgnd; }
        //
        //  PURPOSE:
        //      Return ID of CalAtom
        //



    // Set the required order and level
      const RgsBackgroundServer &order(unsigned inputOrder);
      const RgsBackgroundServer &level(unsigned inputLevel);

    // Return the number of extensions for this order
      const unsigned numberExtensions() const;

    // Return components of the RgsBackgnd object
      const double exposure() const;
      const double bckgndLevel() const;
      const vector<int16> channel() const;
      const vector<int> counts() const;
      const vector<int16> quality() const;
      const vector<float> areascal() const;
      const vector<float> backscal() const;

protected:
     // Calculate the position of the relevant RgsBackgnd object
      const unsigned index() const;

 	private:
    vector<RgsBackgnd *> _background;

    vector<unsigned> _orderArray;
    vector<unsigned> _levelArray;

    bool _levelCalled;
    bool _orderCalled;
    unsigned  _levelRequired;
    unsigned _orderRequired;
    

}; // RgsBackgndServer



#endif
